@@ -24,20 +24,24 @@
 #include "qgsmessagelog.h"
 #include "qgsapplication.h"
 #include "qgsnetworkaccessmanager.h"
+#include "qgsnetworkcontentfetchertask.h"
+#include "qgsvariantutils.h"
 
 #include <QObject>
-#include <QMutex>
+#include <QRecursiveMutex>
 #include <QCache>
 #include <QSet>
 #include <QDateTime>
 #include <QList>
-#include "qgsnetworkcontentfetchertask.h"
+#include <QFile>
 #include <QNetworkReply>
+#include <QFileInfo>
+#include <QUrl>
 
 /**
  * \class QgsAbstractContentCacheEntry
  * \ingroup core
- * Base class for entries in a QgsAbstractContentCache.
+ * \brief Base class for entries in a QgsAbstractContentCache.
  *
  * Subclasses must take care to correctly implement the isEqual() method, applying their
  * own logic for testing extra cache properties (e.g. image size for an image-based cache).
@@ -88,11 +92,7 @@ class CORE_EXPORT QgsAbstractContentCacheEntry
 
     bool operator==( const QgsAbstractContentCacheEntry &other ) const
     {
-      bool equal = other.path == path;
-      if ( equal && ( mFileModifiedCheckTimeout <= 0 || fileModifiedLastCheckTimer.hasExpired( mFileModifiedCheckTimeout ) ) )
-        equal = other.fileModified == fileModified;
-
-      return equal;
+      return other.path == path;
     }
 
     /**
@@ -125,7 +125,7 @@ class CORE_EXPORT QgsAbstractContentCacheEntry
  * \class QgsAbstractContentCacheBase
  * \ingroup core
  *
- * A QObject derived base class for QgsAbstractContentCache.
+ * \brief A QObject derived base class for QgsAbstractContentCache.
  *
  * Required because template based class (such as QgsAbstractContentCache) cannot use the Q_OBJECT macro.
  *
@@ -157,8 +157,8 @@ class CORE_EXPORT QgsAbstractContentCacheBase: public QObject
      */
     virtual bool checkReply( QNetworkReply *reply, const QString &path ) const
     {
-      Q_UNUSED( reply );
-      Q_UNUSED( path );
+      Q_UNUSED( reply )
+      Q_UNUSED( path )
       return true;
     }
 
@@ -167,7 +167,7 @@ class CORE_EXPORT QgsAbstractContentCacheBase: public QObject
     /**
      * Triggered after remote content (i.e. HTTP linked content at the given \a url) has been fetched.
      *
-     * The \a success argument will be true if the content was successfully fetched, or false if
+     * The \a success argument will be TRUE if the content was successfully fetched, or FALSE if
      * it was not fetched successfully.
      */
     virtual void onRemoteContentFetched( const QString &url, bool success );
@@ -180,7 +180,7 @@ class CORE_EXPORT QgsAbstractContentCacheBase: public QObject
  * \class QgsAbstractContentCache
  * \ingroup core
  *
- * Abstract base class for file content caches, such as SVG or raster image caches.
+ * \brief Abstract base class for file content caches, such as SVG or raster image caches.
  *
  * Handles trimming the maximum cached content size to a desired limit, fetching remote
  * content (via HTTP), and automatically invalidating cached content when the corresponding
@@ -211,7 +211,6 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
                              long maxCacheSize = 20000000,
                              int fileModifiedCheckTimeout = 30000 )
       : QgsAbstractContentCacheBase( parent )
-      , mMutex( QMutex::Recursive )
       , mMaxCacheSize( maxCacheSize )
       , mFileModifiedCheckTimeout( fileModifiedCheckTimeout )
       , mTypeString( typeString.isEmpty() ? QObject::tr( "Content" ) : typeString )
@@ -256,8 +255,12 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
      * The \a missingContent byte array is returned if the \a path could not be resolved or is broken. If
      * the \a path corresponds to a remote URL, then \a fetchingContent will be returned while the content
      * is in the process of being fetched.
+     * The \a blocking boolean forces to wait for loading before returning result. The content is loaded
+     * in the same thread to ensure provided the remote content. WARNING: the \a blocking parameter must NEVER
+     * be TRUE from GUI based applications (like the main QGIS application) or crashes will result. Only for
+     * use in external scripts or QGIS server.
      */
-    QByteArray getContent( const QString &path, const QByteArray &missingContent, const QByteArray &fetchingContent ) const
+    QByteArray getContent( const QString &path, const QByteArray &missingContent, const QByteArray &fetchingContent, bool blocking = false ) const
     {
       // is it a path to local file?
       QFile file( path );
@@ -276,7 +279,7 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
       // maybe it's an embedded base64 string
       if ( path.startsWith( QLatin1String( "base64:" ), Qt::CaseInsensitive ) )
       {
-        QByteArray base64 = path.mid( 7 ).toLocal8Bit(); // strip 'base64:' prefix
+        const QByteArray base64 = path.mid( 7 ).toLocal8Bit(); // strip 'base64:' prefix
         return QByteArray::fromBase64( base64, QByteArray::OmitTrailingEquals );
       }
 
@@ -286,7 +289,7 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
         return missingContent;
       }
 
-      QUrl url( path );
+      const QUrl url( path );
       if ( !url.isValid() )
       {
         return missingContent;
@@ -308,11 +311,47 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
         return missingContent;
       }
 
-      QMutexLocker locker( &mMutex );
+      const QMutexLocker locker( &mMutex );
 
       // already a request in progress for this url
       if ( mPendingRemoteUrls.contains( path ) )
-        return fetchingContent;
+      {
+        // it's a non blocking request so return fetching content
+        if ( !blocking )
+        {
+          return fetchingContent;
+        }
+
+        // it's a blocking request so try to find the task and wait for task finished
+        const auto constActiveTasks = QgsApplication::taskManager()->activeTasks();
+        for ( QgsTask *task : constActiveTasks )
+        {
+          // the network content fetcher task's description ends with the path
+          if ( !task->description().endsWith( path ) )
+          {
+            continue;
+          }
+
+          // cast task to network content fetcher task
+          QgsNetworkContentFetcherTask *ncfTask = qobject_cast<QgsNetworkContentFetcherTask *>( task );
+          if ( ncfTask )
+          {
+            // wait for task finished
+            if ( waitForTaskFinished( ncfTask ) )
+            {
+              if ( mRemoteContentCache.contains( path ) )
+              {
+                // We got the file!
+                return *mRemoteContentCache[ path ];
+              }
+            }
+          }
+          // task found, no needs to continue
+          break;
+        }
+        // if no content returns the content is probably in remote content cache
+        // or a new task will be created
+      }
 
       if ( mRemoteContentCache.contains( path ) )
       {
@@ -330,7 +369,7 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
       QgsNetworkContentFetcherTask *task = new QgsNetworkContentFetcherTask( request );
       connect( task, &QgsNetworkContentFetcherTask::fetched, this, [this, task, path, missingContent]
       {
-        QMutexLocker locker( &mMutex );
+        const QMutexLocker locker( &mMutex );
 
         QNetworkReply *reply = task->reply();
         if ( !reply )
@@ -348,10 +387,10 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
 
         bool ok = true;
 
-        QVariant status = reply->attribute( QNetworkRequest::HttpStatusCodeAttribute );
-        if ( !status.isNull() && status.toInt() >= 400 )
+        const QVariant status = reply->attribute( QNetworkRequest::HttpStatusCodeAttribute );
+        if ( !QgsVariantUtils::isNull( status ) && status.toInt() >= 400 )
         {
-          QVariant phrase = reply->attribute( QNetworkRequest::HttpReasonPhraseAttribute );
+          const QVariant phrase = reply->attribute( QNetworkRequest::HttpReasonPhraseAttribute );
           QgsMessageLog::logMessage( tr( "%4 request error [status: %1 - reason phrase: %2] for %3" ).arg( status.toInt() ).arg( phrase.toString(), path, mTypeString ), mTypeString );
           mRemoteContentCache.insert( path, new QByteArray( missingContent ) );
           ok = false;
@@ -366,18 +405,36 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
         if ( ok )
         {
           // read the content data
-          mRemoteContentCache.insert( path, new QByteArray( reply->readAll() ) );
+          const QByteArray ba = reply->readAll();
+
+          // because of the fragility listed below in waitForTaskFinished, this slot may get called twice. In that case
+          // the second time will have an empty reply (we've already read it all...)
+          if ( !ba.isEmpty() )
+            mRemoteContentCache.insert( path, new QByteArray( ba ) );
         }
         QMetaObject::invokeMethod( const_cast< QgsAbstractContentCacheBase * >( qobject_cast< const QgsAbstractContentCacheBase * >( this ) ), "onRemoteContentFetched", Qt::QueuedConnection, Q_ARG( QString, path ), Q_ARG( bool, true ) );
       } );
 
       QgsApplication::taskManager()->addTask( task );
+
+      // if blocking, wait for finished
+      if ( blocking )
+      {
+        if ( waitForTaskFinished( task ) )
+        {
+          if ( mRemoteContentCache.contains( path ) )
+          {
+            // We got the file!
+            return *mRemoteContentCache[ path ];
+          }
+        }
+      }
       return fetchingContent;
     }
 
     void onRemoteContentFetched( const QString &url, bool success ) override
     {
-      QMutexLocker locker( &mMutex );
+      const QMutexLocker locker( &mMutex );
       mPendingRemoteUrls.remove( url );
 
       T *nextEntry = mLeastRecentEntry;
@@ -397,6 +454,35 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
         emit remoteContentFetched( url );
     }
 
+    /**
+     * Blocks the current thread until the \a task finishes (or user's preset network timeout expires)
+     *
+     * \warning this method must NEVER be used from GUI based applications (like the main QGIS application)
+     * or crashes will result. Only for use in external scripts or QGIS server.
+     *
+     * The result will be FALSE if the wait timed out and TRUE in any other case.
+     *
+     * \since QGIS 3.10
+     */
+    bool waitForTaskFinished( QgsNetworkContentFetcherTask *task ) const
+    {
+      // Wait up to timeout seconds for task finished
+      if ( task->waitForFinished( QgsNetworkAccessManager::timeout() ) )
+      {
+        // The wait did not time out
+        // Third step, check status as complete
+        if ( task->status() == QgsTask::Complete )
+        {
+          // Fourth step, force the signal fetched to be sure reply has been checked
+
+          // ARGH this is BAD BAD BAD. The connection will get called twice as a result!!!
+          task->fetched();
+          return true;
+        }
+      }
+      return false;
+    }
+
     /**
      * Returns the existing entry from the cache which matches \a entryTemplate (deleting entryTemplate when done), or
      * if no existing entry is found then \a entryTemplate is transferred to the cache and returned.
@@ -424,6 +510,8 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
 
             if ( cacheEntry->fileModified != modified )
               continue;
+            else
+              cacheEntry->fileModifiedLastCheckTimer.restart();
           }
           currentEntry = cacheEntry;
           break;
@@ -439,6 +527,7 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
       {
         delete entryTemplate;
         entryTemplate = nullptr;
+        ( void )entryTemplate;
         takeEntryFromList( currentEntry );
         if ( !mMostRecentEntry ) //list is empty
         {
@@ -459,8 +548,8 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
 
       return currentEntry;
     }
+    mutable QRecursiveMutex mMutex;
 
-    mutable QMutex mMutex;
     //! Estimated total size of all cached content
     long mTotalSize = 0;
 
@@ -478,6 +567,12 @@ class CORE_EXPORT QgsAbstractContentCache : public QgsAbstractContentCacheBase
     {
       entry->mFileModifiedCheckTimeout = mFileModifiedCheckTimeout;
 
+      if ( !entry->path.startsWith( QLatin1String( "base64:" ) ) )
+      {
+        entry->fileModified = QFileInfo( entry->path ).lastModified();
+        entry->fileModifiedLastCheckTimer.start();
+      }
+
       mEntryLookup.insert( entry->path, entry );
 
       //insert to most recent place in entry list
