@@ -9,21 +9,21 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.28.3-Firenze"
-#define _QGIS_VERSION "3.28.3-Firenze"
+#define VERSION "3.30.0-'s-Hertogenbosch"
+#define _QGIS_VERSION "3.30.0-'s-Hertogenbosch"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3283
+//#define VERSION_INT 3300
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 32803
-#define _QGIS_VERSION_INT 32803
-#define ABISYM(x) x ## 32803
+#define VERSION_INT 33000
+#define _QGIS_VERSION_INT 33000
+#define ABISYM(x) x ## 33000
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Firenze"
+#define RELEASE_NAME "'s-Hertogenbosch"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -31,7 +31,7 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.28.3"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.30.0"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
@@ -46,10 +46,10 @@
 
 #define QT_PLUGINS_DIR "/usr/lib/qt/plugins"
 
-#define PYTHON_VERSION "3.10.9"
+#define PYTHON_VERSION "3.10.10"
 #define PYTHON_VERSION_MAJOR "3"
 #define PYTHON_VERSION_MINOR "10"
-#define PYTHON_VERSION_PATCH "9"
+#define PYTHON_VERSION_PATCH "10"
 
 #define PROJ_VERSION_MAJOR 9
 #define PROJ_VERSION_MINOR 1
@@ -90,9 +90,11 @@
 
 /* #undef QGISDEBUG */
 
+/* #undef AGGRESSIVE_SAFE_MODE */
+
 #define HAVE_QUICK
 
-#define HAVE_QT5SERIALPORT
+#define HAVE_QTSERIALPORT
 
 /* #undef HAVE_STATIC_PROVIDERS */
 
@@ -101,12 +103,12 @@
 #define HAVE_COPC
 
 #define HAVE_PDAL_QGIS
-#define PDAL_VERSION "2.5.0"
+#define PDAL_VERSION "2.5.2"
 #define PDAL_VERSION_MAJOR_INT 2
 #define PDAL_VERSION_MINOR_INT 5
 #define PDAL_VERSION_MAJOR "2"
 #define PDAL_VERSION_MINOR "5"
-#define PDAL_VERSION_MICRO "0"
+#define PDAL_VERSION_MICRO "2"
 
 #define ENABLE_TESTS
 
