@@ -24,7 +24,6 @@
 #include "qgsrendercontext.h"
 #include "qgslayoutundocommand.h"
 #include "qgslayoutmeasurement.h"
-#include "qgsapplication.h"
 #include <QGraphicsRectItem>
 #include <QIcon>
 #include <QPainter>
@@ -38,7 +37,7 @@ class QgsStyleEntityVisitorInterface;
 /**
  * \ingroup core
  * \class QgsLayoutItemRenderContext
- * Contains settings and helpers relating to a render of a QgsLayoutItem.
+ * \brief Contains settings and helpers relating to a render of a QgsLayoutItem.
  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsLayoutItemRenderContext
@@ -123,6 +122,8 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
 #include "qgslayoutframe.h"
 #include "qgslayoutitemshape.h"
 #include "qgslayoutitempage.h"
+#include "qgslayoutitemmarker.h"
+#include "qgslayoutitemelevationprofile.h"
 #endif
 
 #ifdef SIP_RUN
@@ -182,6 +183,14 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
         sipType = sipType_QgsLayoutFrame;
         *sipCppRet = static_cast<QgsLayoutFrame *>( sipCpp );
         break;
+      case QGraphicsItem::UserType + 117:
+        sipType = sipType_QgsLayoutItemMarker;
+        *sipCppRet = static_cast<QgsLayoutItemMarker *>( sipCpp );
+        break;
+      case QGraphicsItem::UserType + 118:
+        sipType = sipType_QgsLayoutItemElevationProfile;
+        *sipCppRet = static_cast<QgsLayoutItemElevationProfile *>( sipCpp );
+        break;
 
       // did you read that comment above? NO? Go read it now. You're about to break stuff.
 
@@ -221,6 +230,10 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
       UndoStrokeWidth, //!< Stroke width adjustment
       UndoBackgroundColor, //!< Background color adjustment
       UndoOpacity, //!< Opacity adjustment
+      UndoMarginLeft, //!< Left margin (since QGIS 3.30)
+      UndoMarginTop, //!< Top margin (since QGIS 3.30)
+      UndoMarginBottom, //!< Bottom margin (since QGIS 3.30)
+      UndoMarginRight, //!< Right margin (since QGIS 3.30)
       UndoSetId, //!< Change item ID
       UndoRotation, //!< Rotation adjustment
       UndoShapeStyle, //!< Shape symbol style
@@ -258,6 +271,8 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
       UndoLegendWmsLegendHeight, //!< Legend WMS height
       UndoLegendTitleSpaceBottom, //!< Legend title space
       UndoLegendGroupSpace, //!< Legend group spacing
+      UndoLegendGroupIndent, //!< Legend group indent
+      UndoLegendSubgroupIndent, //!< Legend subgroup indent
       UndoLegendLayerSpace, //!< Legend layer spacing
       UndoLegendSymbolSpace, //!< Legend symbol spacing
       UndoLegendIconSymbolSpace, //!< Legend icon symbol space
@@ -290,6 +305,23 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
       UndoArrowHeadWidth, //!< Arrow head width
       UndoArrowHeadFillColor, //!< Arrow head fill color
       UndoArrowHeadStrokeColor, //!< Arrow head stroke color
+      UndoElevationProfileTolerance, //!< Change elevation profile distance tolerance
+      UndoElevationProfileChartBackground, //!< Change elevation profile chart background
+      UndoElevationProfileChartBorder, //!< Change elevation profile chart border
+      UndoElevationProfileDistanceMajorGridlines, //!< Change elevation profile distance axis major gridlines
+      UndoElevationProfileDistanceMinorGridlines, //!< Change elevation profile distance axis minor gridlines
+      UndoElevationProfileDistanceFormat, //!< Change elevation profile distance axis number format
+      UndoElevationProfileDistanceFont, //!< Change elevation profile distance axis number font
+      UndoElevationProfileDistanceLabels, //!< Change elevation profile distance axis label interval
+      UndoElevationProfileElevationMajorGridlines, //!< Change elevation profile elevation axis major gridlines
+      UndoElevationProfileElevationMinorGridlines, //!< Change elevation profile elevation axis minor gridlines
+      UndoElevationProfileElevationFormat, //!< Change elevation profile elevation axis number format
+      UndoElevationProfileElevationFont, //!< Change elevation profile elevation axis number font
+      UndoElevationProfileElevationLabels, //!< Change elevation profile elevation axis label interval
+      UndoElevationProfileMinimumDistance, //!< Change elevation profile minimum distance
+      UndoElevationProfileMaximumDistance, //!< Change elevation profile maximum distance
+      UndoElevationProfileMinimumElevation, //!< Change elevation profile minimum elevation
+      UndoElevationProfileMaximumElevation, //!< Change elevation profile maximum elevation
 
       UndoCustomCommand, //!< Base id for plugin based item undo commands
     };
@@ -331,7 +363,7 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
     /**
      * Returns the item's icon.
      */
-    virtual QIcon icon() const { return QgsApplication::getThemeIcon( QStringLiteral( "/mLayoutItem.svg" ) ); }
+    virtual QIcon icon() const;
 
     /**
      * Returns the item identification string. This is a unique random string set for the item
@@ -483,7 +515,7 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
     virtual bool nextExportPart();
 
     /**
-     * Contains details of a particular export layer relating to a layout item.
+     * \brief Contains details of a particular export layer relating to a layout item.
      * \ingroup core
      * \since QGIS 3.10
      */
@@ -1285,7 +1317,7 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
     //! Item frame color
     QColor mFrameColor = QColor( 0, 0, 0 );
     //! Item frame width
-    QgsLayoutMeasurement mFrameWidth = QgsLayoutMeasurement( 0.3, QgsUnitTypes::LayoutMillimeters );
+    QgsLayoutMeasurement mFrameWidth = QgsLayoutMeasurement( 0.3, Qgis::LayoutUnit::Millimeters );
     //! Frame join style
     Qt::PenJoinStyle mFrameJoinStyle = Qt::MiterJoin;
 
