@@ -33,7 +33,7 @@
 
 #include "qgscolorrampshader.h"
 #include "qgsdataprovider.h"
-#include "qgsraster.h"
+#include "qgsrasterattributetable.h"
 #include "qgsfields.h"
 #include "qgsrasterinterface.h"
 #include "qgsrasterpyramid.h"
@@ -83,7 +83,7 @@ class CORE_EXPORT QgsImageFetcher : public QObject
 
 /**
  * \ingroup core
- * Base class for raster data providers.
+ * \brief Base class for raster data providers.
  */
 class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRasterInterface
 {
@@ -101,7 +101,10 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
       ReadLayerMetadata = 1 << 1, //!< Provider can read layer metadata from data store. Since QGIS 3.0. See QgsDataProvider::layerMetadata()
       WriteLayerMetadata = 1 << 2, //!< Provider can write layer metadata to the data store. Since QGIS 3.0. See QgsDataProvider::writeLayerMetadata()
       ProviderHintBenefitsFromResampling = 1 << 3, //!< Provider benefits from resampling and should apply user default resampling settings (since QGIS 3.10)
-      ProviderHintCanPerformProviderResampling = 1 << 4 //!< Provider can perform resampling (to be opposed to post rendering resampling) (since QGIS 3.16)
+      ProviderHintCanPerformProviderResampling = 1 << 4, //!< Provider can perform resampling (to be opposed to post rendering resampling) (since QGIS 3.16)
+      ReloadData = 1 << 5, //!< Is able to force reload data / clear local caches. Since QGIS 3.18, see QgsDataProvider::reloadProviderData()
+      DpiDependentData = 1 << 6, //! Provider's rendering is dependent on requested pixel size of the viewport (since QGIS 3.20)
+      NativeRasterAttributeTable = 1 << 7, //!< Indicates that the provider supports native raster attribute table (since QGIS 3.30)
     };
 
     //! Provider capabilities
@@ -151,79 +154,77 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
     Qgis::DataType sourceDataType( int bandNo ) const override = 0;
 
     //! Returns data type for the band specified by number
-    virtual int colorInterpretation( int bandNo ) const
-    {
-      Q_UNUSED( bandNo )
-      return QgsRaster::UndefinedColorInterpretation;
-    }
+    virtual Qgis::RasterColorInterpretation colorInterpretation( int bandNo ) const;
 
-    QString colorName( int colorInterpretation ) const
+    /**
+     * Returns a string color name representation of a color interpretation.
+     */
+    QString colorName( Qgis::RasterColorInterpretation colorInterpretation ) const
     {
       // Modified copy from GDAL
       switch ( colorInterpretation )
       {
-        case QgsRaster::UndefinedColorInterpretation:
+        case Qgis::RasterColorInterpretation::Undefined:
           return QStringLiteral( "Undefined" );
 
-        case QgsRaster::GrayIndex:
+        case Qgis::RasterColorInterpretation::GrayIndex:
           return QStringLiteral( "Gray" );
 
-        case QgsRaster::PaletteIndex:
+        case Qgis::RasterColorInterpretation::PaletteIndex:
           return QStringLiteral( "Palette" );
 
-        case QgsRaster::RedBand:
+        case Qgis::RasterColorInterpretation::RedBand:
           return QStringLiteral( "Red" );
 
-        case QgsRaster::GreenBand:
+        case Qgis::RasterColorInterpretation::GreenBand:
           return QStringLiteral( "Green" );
 
-        case QgsRaster::BlueBand:
+        case Qgis::RasterColorInterpretation::BlueBand:
           return QStringLiteral( "Blue" );
 
-        case QgsRaster::AlphaBand:
+        case Qgis::RasterColorInterpretation::AlphaBand:
           return QStringLiteral( "Alpha" );
 
-        case QgsRaster::HueBand:
+        case Qgis::RasterColorInterpretation::HueBand:
           return QStringLiteral( "Hue" );
 
-        case QgsRaster::SaturationBand:
+        case Qgis::RasterColorInterpretation::SaturationBand:
           return QStringLiteral( "Saturation" );
 
-        case QgsRaster::LightnessBand:
+        case Qgis::RasterColorInterpretation::LightnessBand:
           return QStringLiteral( "Lightness" );
 
-        case QgsRaster::CyanBand:
+        case Qgis::RasterColorInterpretation::CyanBand:
           return QStringLiteral( "Cyan" );
 
-        case QgsRaster::MagentaBand:
+        case Qgis::RasterColorInterpretation::MagentaBand:
           return QStringLiteral( "Magenta" );
 
-        case QgsRaster::YellowBand:
+        case Qgis::RasterColorInterpretation::YellowBand:
           return QStringLiteral( "Yellow" );
 
-        case QgsRaster::BlackBand:
+        case Qgis::RasterColorInterpretation::BlackBand:
           return QStringLiteral( "Black" );
 
-        case QgsRaster::YCbCr_YBand:
+        case Qgis::RasterColorInterpretation::YCbCr_YBand:
           return QStringLiteral( "YCbCr_Y" );
 
-        case QgsRaster::YCbCr_CbBand:
+        case Qgis::RasterColorInterpretation::YCbCr_CbBand:
           return QStringLiteral( "YCbCr_Cb" );
 
-        case QgsRaster::YCbCr_CrBand:
+        case Qgis::RasterColorInterpretation::YCbCr_CrBand:
           return QStringLiteral( "YCbCr_Cr" );
 
-        default:
-          return QStringLiteral( "Unknown" );
+        case Qgis::RasterColorInterpretation::ContinuousPalette:
+          return QStringLiteral( "Continuous Palette" );
       }
+      return QString();
     }
+
     //! Reload data (data could change)
     virtual bool reload() { return true; }
 
-    virtual QString colorInterpretationName( int bandNo ) const
-    {
-      return colorName( colorInterpretation( bandNo ) );
-    }
+    QString colorInterpretationName( int bandNo ) const override;
 
     /**
      * Read band scale for raster value
@@ -315,10 +316,25 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
       return nullptr;
     }
 
-    //! \brief Create pyramid overviews
+    /**
+     * Creates pyramid overviews.
+     *
+     * \param pyramidList a list of QgsRasterPyramids to create overviews for. The QgsRasterPyramid::setBuild() flag
+     * should be set to TRUE for every layer where pyramids are desired.
+     * \param resamplingMethod resampling method to use when creating the pyramids. The pyramidResamplingMethods() method
+     * can be used to retrieve a list of valid resampling methods available for specific raster data providers.
+     * \param format raster pyramid format.
+     * \param configOptions optional configuration options which are passed to the specific data provider
+     * for use during pyramid creation.
+     * \param feedback optional feedback argument for progress reports and cancellation support.
+     *
+     * \see buildPyramidList()
+     * \see hasPyramids()
+     * \see pyramidResamplingMethods()
+     */
     virtual QString buildPyramids( const QList<QgsRasterPyramid> &pyramidList,
                                    const QString &resamplingMethod = "NEAREST",
-                                   QgsRaster::RasterPyramidsFormat format = QgsRaster::PyramidsGTiff,
+                                   Qgis::RasterPyramidFormat format = Qgis::RasterPyramidFormat::GeoTiff,
                                    const QStringList &configOptions = QStringList(),
                                    QgsRasterBlockFeedback *feedback = nullptr )
     {
@@ -332,16 +348,34 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     /**
      * Returns the raster layers pyramid list.
-     * \param overviewList used to construct the pyramid list (optional), when empty the list is defined by the provider.
-     * A pyramid list defines the
-     * POTENTIAL pyramids that can be in a raster. To know which of the pyramid layers
-     * ACTUALLY exists you need to look at the existsFlag member in each struct stored in the
+     *
+     * This method returns a list of pyramid layers which are valid for the data provider. The returned list
+     * is a complete list of all possible layers, and includes both pyramids layers which currently exist and
+     * layers which have not yet been constructed. To know which of the pyramid layers
+     * ACTUALLY exists you need to look at the QgsRasterPyramid::getExists() member for each value in the
      * list.
+     *
+     * The returned list is suitable for passing to the buildPyramids() method. First, modify the returned list
+     * by calling `QgsRasterPyramid::setBuild( TRUE )` for every layer you want to create pyramids for, and then
+     * pass the modified list to buildPyramids().
+     *
+     * \param overviewList used to construct the pyramid list (optional), when empty the list is defined by the provider.
+     *
+     * \see buildPyramids()
+     * \see hasPyramids()
      */
-    virtual QList<QgsRasterPyramid> buildPyramidList( QList<int> overviewList = QList<int>() ) // clazy:exclude=function-args-by-ref
+    virtual QList<QgsRasterPyramid> buildPyramidList( const QList<int> &overviewList = QList<int>() )
     { Q_UNUSED( overviewList ) return QList<QgsRasterPyramid>(); }
 
-    //! \brief Returns TRUE if raster has at least one populated histogram.
+    /**
+     * Returns TRUE if raster has at least one existing pyramid.
+     *
+     * The buildPyramidList() method can be used to retrieve additional details about potential and existing
+     * pyramid layers.
+     *
+     * \see buildPyramidList()
+     * \see buildPyramids()
+     */
     bool hasPyramids();
 
     /**
@@ -374,7 +408,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      *       up to at least v1.3.0
      * \see sample(), which is much more efficient for simple "value at point" queries.
      */
-    virtual QgsRasterIdentifyResult identify( const QgsPointXY &point, QgsRaster::IdentifyFormat format, const QgsRectangle &boundingBox = QgsRectangle(), int width = 0, int height = 0, int dpi = 96 );
+    virtual QgsRasterIdentifyResult identify( const QgsPointXY &point, Qgis::RasterIdentifyFormat format, const QgsRectangle &boundingBox = QgsRectangle(), int width = 0, int height = 0, int dpi = 96 );
 
     /**
      * Samples a raster value from the specified \a band found at the \a point position. The context
@@ -507,6 +541,48 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     static QList<QPair<QString, QString> > pyramidResamplingMethods( const QString &providerKey );
 
+    /**
+     * Struct that stores information of the raster used in QgsVirtualRasterProvider for the calculations,
+     * this struct is  stored in the DecodedUriParameters
+     * \note used by QgsVirtualRasterProvider only
+     */
+    struct VirtualRasterInputLayers
+    {
+      QString name;
+      QString uri;
+      QString provider;
+    };
+
+    /**
+     * Struct that stores the information about the parameters that should be given to the
+     * QgsVirtualRasterProvider through the QgsRasterDataProvider::DecodedUriParameters
+     * \note used by QgsVirtualRasterProvider only
+     */
+    struct VirtualRasterParameters
+    {
+      QgsCoordinateReferenceSystem crs;
+      QgsRectangle extent;
+      int width;
+      int height;
+      QString formula;
+      QList <QgsRasterDataProvider::VirtualRasterInputLayers> rInputLayers;
+
+    };
+
+    /**
+     * Decodes the URI returning a struct with all the parameters for QgsVirtualRasterProvider class
+     * \note used by Virtual Raster Provider only
+     * \note since QGIS 3.22
+     */
+    static QgsRasterDataProvider::VirtualRasterParameters decodeVirtualRasterProviderUri( const QString &uri, bool *ok = nullptr );
+
+    /**
+     * Encodes the URI starting from the struct .
+     * \note used by Virtual Raster Provider only
+     * \note since QGIS 3.22
+     */
+    static QString encodeVirtualRasterProviderUri( const VirtualRasterParameters &parts );
+
     /**
      * Validates creation options for a specific dataset and destination format.
      * \note used by GDAL provider only
@@ -519,14 +595,33 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      * Validates pyramid creation options for a specific dataset and destination format
      * \note used by GDAL provider only
      */
-    virtual QString validatePyramidsConfigOptions( QgsRaster::RasterPyramidsFormat pyramidsFormat,
+    virtual QString validatePyramidsConfigOptions( Qgis::RasterPyramidFormat pyramidsFormat,
         const QStringList &configOptions, const QString &fileFormat )
     { Q_UNUSED( pyramidsFormat ) Q_UNUSED( configOptions ); Q_UNUSED( fileFormat ); return QString(); }
 
-    static QString identifyFormatName( QgsRaster::IdentifyFormat format );
-    static QgsRaster::IdentifyFormat identifyFormatFromName( const QString &formatName );
-    static QString identifyFormatLabel( QgsRaster::IdentifyFormat format );
-    static Capability identifyFormatToCapability( QgsRaster::IdentifyFormat format );
+    /**
+     * Converts a raster identify \a format to a string name.
+     *
+     * \see identifyFormatFromName()
+     */
+    static QString identifyFormatName( Qgis::RasterIdentifyFormat format );
+
+    /**
+     * Converts a string \a formatName to a raster identify format.
+     *
+     * \see identifyFormatName()
+     */
+    static Qgis::RasterIdentifyFormat identifyFormatFromName( const QString &formatName );
+
+    /**
+     * Converts a raster identify \a format to a translated string label.
+     */
+    static QString identifyFormatLabel( Qgis::RasterIdentifyFormat format );
+
+    /**
+     * Converts a raster identify \a format to a capability.
+     */
+    static Capability identifyFormatToCapability( Qgis::RasterIdentifyFormat format );
 
     /**
      * Step width for raster iterations.
@@ -613,9 +708,14 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     enum class ResamplingMethod
     {
-      Nearest,      //!< Nearest-neighbour resamplikng
-      Bilinear,     //!< Bilinear resamplikng
-      Cubic,     //!< Bicubic resamplikng
+      Nearest, //!< Nearest-neighbour resampling
+      Bilinear, //!< Bilinear (2x2 kernel) resampling
+      Cubic,//!< Cubic Convolution Approximation (4x4 kernel) resampling
+      CubicSpline, //!< Cubic B-Spline Approximation (4x4 kernel)
+      Lanczos, //!< Lanczos windowed sinc interpolation (6x6 kernel)
+      Average, //!< Average resampling
+      Mode, //!< Mode (selects the value which appears most often of all the sampled points)
+      Gauss //!< Gauss blurring
     };
 
     /**
@@ -664,6 +764,71 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     void writeXml( QDomDocument &doc, QDomElement &parentElem ) const override;
 
+    /**
+     * Returns the (possibly NULL) attribute table for the specified \a bandNumber.
+     *
+     * \since QGIS 3.30
+     */
+    QgsRasterAttributeTable *attributeTable( int bandNumber ) const;
+
+    /**
+     * Set the attribute table to \a attributeTable for the specified \a bandNumber,
+     * if the \a attributeTable is NULL any existing attribute table for the specified
+     * band will be removed.
+     *
+     * \note Ownership of the attribute table is transferred to the provider.
+     * \since QGIS 3.30
+     */
+    void setAttributeTable( int bandNumber, QgsRasterAttributeTable *attributeTable SIP_TRANSFER );
+
+    /**
+     * Remove the attribute table for the specified \a bandNumber.
+     * If the attribute table does not exist this method does nothing.
+     *
+     * \since QGIS 3.30
+     */
+    void removeAttributeTable( int bandNumber );
+
+    /**
+     * Writes the filesystem-based attribute table for the specified \a bandNumber to \a path, optionally reporting any error in \a errorMessage, returns TRUE on success.
+     *
+     * \returns TRUE on success
+     * \note No checks for Raster Attribute Table validity are performed when saving, it is client code responsibility to handle validation.
+     * \since QGIS 3.30
+     */
+    bool writeFileBasedAttributeTable( int bandNumber, const QString &path, QString *errorMessage SIP_OUT = nullptr ) const;
+
+    /**
+     * Loads the filesystem-based attribute table for the specified \a bandNumber from \a path, optionally reporting any error in \a errorMessage, returns TRUE on success.
+     *
+     * \returns TRUE on success
+     * \since QGIS 3.30
+     */
+    bool readFileBasedAttributeTable( int bandNumber, const QString &path, QString *errorMessage SIP_OUT = nullptr );
+
+    /**
+     * Writes the native attribute table, optionally reporting any error in \a errorMessage, returns TRUE on success.
+     * The default implementation does nothing and returns FALSE.
+     * Data providers that have NativeRasterAttributeTable
+     * provider capability will try to save the native attribute table.
+     *
+     * \returns TRUE on success
+     * \note No checks for Raster Attribute Table validity are performed when saving, it is client code responsibility to handle validation.
+     * \since QGIS 3.30
+     */
+    virtual bool writeNativeAttributeTable( QString *errorMessage SIP_OUT = nullptr );  //#spellok
+
+    /**
+     * Reads the native attribute table, optionally reporting any error in \a errorMessage, returns TRUE on success.
+     * The default implementation does nothing and returns FALSE.
+     * Data providers that have NativeRasterAttributeTable provider capability will try to read the native attribute table.
+     *
+     * \returns TRUE on success
+     * \since QGIS 3.30
+     */
+    virtual bool readNativeAttributeTable( QString *errorMessage SIP_OUT  = nullptr );
+
+
   signals:
 
     /**
@@ -672,6 +837,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     void statusChanged( const QString & ) const;
 
+
   protected:
 
     /**
@@ -698,7 +864,7 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
 
     /**
      * Dots per inch. Extended WMS (e.g. QGIS mapserver) support DPI dependent output and therefore
-    are suited for printing. A value of -1 means it has not been set
+     * are suited for printing. A value of -1 means it has not been set
     */
     int mDpi = -1;
 
@@ -748,6 +914,8 @@ class CORE_EXPORT QgsRasterDataProvider : public QgsDataProvider, public QgsRast
      */
     std::unique_ptr< QgsRasterDataProviderTemporalCapabilities > mTemporalCapabilities;
 
+    std::map<int, std::unique_ptr<QgsRasterAttributeTable>> mAttributeTables;
+
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsRasterDataProvider::ProviderCapabilities )
