@@ -21,17 +21,13 @@ __author__ = 'Alexander Bruy'
 __date__ = 'December 2012'
 __copyright__ = '(C) 2012, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 import os
 import codecs
 import inspect
 import traceback
 import warnings
 
-from qgis.PyQt import uic
+from qgis.PyQt import uic, sip
 from qgis.PyQt.QtCore import Qt
 from qgis.PyQt.QtGui import QCursor
 from qgis.PyQt.QtWidgets import (QMessageBox,
@@ -44,6 +40,7 @@ from qgis.core import (QgsApplication,
                        QgsProcessingAlgorithm,
                        QgsProcessingFeatureBasedAlgorithm)
 from qgis.utils import iface, OverrideCursor
+from qgis.processing import alg as algfactory
 
 from processing.gui.AlgorithmDialog import AlgorithmDialog
 from processing.script import ScriptUtils
@@ -65,7 +62,6 @@ class ScriptEditorDialog(BASE, WIDGET):
 
         QgsGui.instance().enableAutoGeometryRestore(self)
 
-        self.editor.initLexer()
         self.searchWidget.setVisible(False)
 
         if iface is not None:
@@ -96,6 +92,8 @@ class ScriptEditorDialog(BASE, WIDGET):
             QgsApplication.getThemeIcon('/mActionIncreaseFont.svg'))
         self.actionDecreaseFontSize.setIcon(
             QgsApplication.getThemeIcon('/mActionDecreaseFont.svg'))
+        self.actionToggleComment.setIcon(
+            QgsApplication.getThemeIcon('console/iconCommentEditorConsole.svg'))
 
         # Connect signals and slots
         self.actionOpenScript.triggered.connect(self.openScript)
@@ -110,12 +108,14 @@ class ScriptEditorDialog(BASE, WIDGET):
         self.actionFindReplace.toggled.connect(self.toggleSearchBox)
         self.actionIncreaseFontSize.triggered.connect(self.editor.zoomIn)
         self.actionDecreaseFontSize.triggered.connect(self.editor.zoomOut)
+        self.actionToggleComment.triggered.connect(self.editor.toggleComment)
         self.editor.textChanged.connect(lambda: self.setHasChanged(True))
 
         self.leFindText.returnPressed.connect(self.find)
         self.btnFind.clicked.connect(self.find)
         self.btnReplace.clicked.connect(self.replace)
         self.lastSearch = None
+        self.run_dialog = None
 
         self.filePath = None
         if filePath is not None:
@@ -222,9 +222,13 @@ class ScriptEditorDialog(BASE, WIDGET):
         self.update_dialog_title()
 
     def runAlgorithm(self):
-        d = {}
+        if self.run_dialog and not sip.isdeleted(self.run_dialog):
+            self.run_dialog.close()
+            self.run_dialog = None
+
+        _locals = {}
         try:
-            exec(self.editor.text(), d)
+            exec(self.editor.text(), _locals)
         except Exception as e:
             error = QgsError(traceback.format_exc(), "Processing")
             QgsErrorDialog.show(error,
@@ -233,10 +237,13 @@ class ScriptEditorDialog(BASE, WIDGET):
             return
 
         alg = None
-        for k, v in d.items():
-            if inspect.isclass(v) and issubclass(v, (QgsProcessingAlgorithm, QgsProcessingFeatureBasedAlgorithm)) and v.__name__ not in ("QgsProcessingAlgorithm", "QgsProcessingFeatureBasedAlgorithm"):
-                alg = v()
-                break
+        try:
+            alg = algfactory.instances.pop().createInstance()
+        except IndexError:
+            for name, attr in _locals.items():
+                if inspect.isclass(attr) and issubclass(attr, (QgsProcessingAlgorithm, QgsProcessingFeatureBasedAlgorithm)) and attr.__name__ not in ("QgsProcessingAlgorithm", "QgsProcessingFeatureBasedAlgorithm"):
+                    alg = attr()
+                    break
 
         if alg is None:
             QMessageBox.warning(self,
@@ -248,14 +255,14 @@ class ScriptEditorDialog(BASE, WIDGET):
         alg.setProvider(QgsApplication.processingRegistry().providerById("script"))
         alg.initAlgorithm()
 
-        dlg = alg.createCustomParametersWidget(iface.mainWindow())
-        if not dlg:
-            dlg = AlgorithmDialog(alg, parent=iface.mainWindow())
+        self.run_dialog = alg.createCustomParametersWidget(self)
+        if not self.run_dialog:
+            self.run_dialog = AlgorithmDialog(alg, parent=self)
 
         canvas = iface.mapCanvas()
         prevMapTool = canvas.mapTool()
 
-        dlg.show()
+        self.run_dialog.show()
 
         if canvas.mapTool() != prevMapTool:
             try:
